*** WeiDU-185/obj/x86_LINUX/bafparser.ml	1970-01-01 01:00:00.000000000 +0100
--- WeiDU/obj/x86_LINUX/bafparser.ml	2007-07-26 15:35:18.000000000 +0200
***************
*** 0 ****
--- 1,892 ----
+ type token =
+   | LPAREN
+   | RPAREN
+   | LBRACKET
+   | RBRACKET
+   | PERIOD
+   | COMMA
+   | ACTIONOVERRIDE
+   | NOT
+   | EOF
+   | IF
+   | THEN
+   | END
+   | RESPONSE
+   | ANYONE
+   | STRING of (string)
+   | SYMBOL of (string)
+   | TILDE_STRING of (string)
+   | INTEGER of (Int32.t)
+   | TRANS_REF of (Int32.t)
+ 
+ open Parsing;;
+ # 2 "src/bafparser.mly"
+ open Util
+ open Ids
+ open Bcs
+ open Load
+ 
+ (* Note added due to LGPL terms.
+ 
+ This file was edited by Valerio Bigiani, AKA The Bigg, starting from
+ 6 November 2005. All changes for this file are listed in
+ diffs/src.bafparser.mly.diff file, as the output of a diff -Bw -c -N command.
+ 
+ It was originally taken from Westley Weimer's WeiDU 185. *)
+ 
+ 
+ (*** input handle ***)
+ 
+ (*** Error handling ***)
+ 
+ type bafarg = 
+   BA_Symbol of string
+ | BA_Integer of Int32.t
+ | BA_String of string
+ | BA_Bracket of bracket_arg list
+ | BA_Compound of string * bafarg
+ | BA_Rect of bafarg * (Int32.t * Int32.t * Int32.t * Int32.t) option
+ and bracket_arg =
+   Bracket_Integer of Int32.t
+ | Bracket_Symbol of string
+ 
+ let get_next_of_type argl tau = 
+   try 
+     let res = List.assoc tau !argl in
+     argl := List.remove_assoc tau !argl ;
+     res
+   with e -> begin
+           match tau with
+           | Arg_Integer -> Act_Integer Int32.zero
+           | Arg_String -> Act_String ""
+           | Arg_Object -> Act_Object (empty_object_param ())
+           | Arg_Point -> Act_Point(Int32.zero,Int32.zero)
+           | Arg_Action -> failwith "arg_action"
+           end
+ 
+ let get_next_int argl =
+   match (get_next_of_type argl Arg_Integer) with
+     Act_Integer i -> i
+   | _ -> failwith "get_next_int"
+ let get_next_string argl =
+   match (get_next_of_type argl Arg_String) with
+     Act_String i -> i
+   | _ -> failwith "get_next_string"
+ let get_next_object argl =
+   match (get_next_of_type argl Arg_Object) with
+     Act_Object i -> i
+   | _ -> failwith "get_next_object"
+ let get_next_point argl =
+   match (get_next_of_type argl Arg_Point) with
+     Act_Point i -> i
+   | _ -> failwith "get_next_point"
+ 
+ let valid_var_area s =
+   let s = String.uppercase s in
+   if !debug_ocaml then log_and_print "valid_var_area %s\n" s;
+   let ans = (s = "GLOBAL" || s = "LOCALS" || s = "KAPUTZ" || s = "MYAREA" || s = "" ||
+   ( Str.string_match (Str.regexp "AR[0-9][0-9][0-9][0-9]") s 0) || file_exists_in_game (the_game ()) (s ^ ".ARE")) in
+   ans || not !debug_modder
+ 
+ let fixup_concat argl =
+   let s1 = get_next_string argl in
+   let s2 = get_next_string argl in
+   if not (valid_var_area s2) then log_and_print "This expression has a typo in the second part: Global(\"%s\",\"%s\")\n" s1 s2;
+   argl := (Arg_String,Act_String(s2^s1)) :: !argl
+ 
+ let fixup_concat2 argl =
+   let s1 = get_next_string argl in
+   let s2 = get_next_string argl in
+   if not (valid_var_area s2) then log_and_print "This expression has a typo in the second part: *Global*(\"%s\",\"%s\")\n" s1 s2;
+   let s3 = get_next_string argl in
+   let s4 = get_next_string argl in
+   if not (valid_var_area s4) then log_and_print "This expression has a typo in the second part: *Global*(\"%s\",\"%s\")\n" s3 s4;
+   argl := (Arg_String,Act_String(s2^s1)) ::
+           (Arg_String,Act_String(s4^s3)) :: !argl
+ 
+ let assign_bracket x pos obj ss =
+   match ss, pos with
+   | Load.IWD2,0 -> obj.o_ea <- x
+   | Load.IWD2,1 -> obj.o_general <- x
+   | Load.IWD2,2 -> obj.o_race <- x
+   | Load.IWD2,3 -> obj.o_subrace <- x
+   | Load.IWD2,4 -> obj.o_specific <- x
+   | Load.IWD2,5 -> obj.o_gender <- x
+   | Load.IWD2,6 -> obj.o_alignment <- x
+   | Load.IWD2,7 -> obj.o_class <- x
+   | Load.IWD2,8 -> obj.o_iwd2_1 <- x
+   | Load.IWD2,9 -> obj.o_iwd2_2 <- x
+ 
+   | Load.IWD1,0 -> obj.o_ea <- x
+   | Load.IWD1,1 -> obj.o_general <- x
+   | Load.IWD1,2 -> obj.o_race <- x
+   | Load.IWD1,3 -> obj.o_class <- x
+   | Load.IWD1,4 -> obj.o_specific <- x
+   | Load.IWD1,5 -> obj.o_gender <- x
+   | Load.IWD1,6 -> obj.o_alignment <- x
+ 
+   | Load.BG,0 -> obj.o_ea <- x
+   | Load.BG,1 -> obj.o_general <- x
+   | Load.BG,2 -> obj.o_race <- x
+   | Load.BG,3 -> obj.o_class <- x
+   | Load.BG,4 -> obj.o_specific <- x
+   | Load.BG,5 -> obj.o_gender <- x
+   | Load.BG,6 -> obj.o_alignment <- x
+ 
+   | Load.PST,0 -> obj.o_ea <- x
+   | Load.PST,1 -> obj.o_faction <- x
+   | Load.PST,2 -> obj.o_team <- x
+   | Load.PST,3 -> obj.o_general <- x
+   | Load.PST,4 -> obj.o_race <- x
+   | Load.PST,5 -> obj.o_class <- x
+   | Load.PST,6 -> obj.o_specific <- x
+   | Load.PST,7 -> obj.o_gender <- x
+   | Load.PST,8 -> obj.o_alignment <- x
+ 
+   | _ -> failwith (Printf.sprintf "assign %d to unknown bracket position %d" (Int32.to_int x) pos)
+ 
+ let assign_compound x pos obj = match pos with
+   | 0 -> obj.o_unknown4 <- x
+   | 1 -> obj.o_unknown3 <- x
+   | 2 -> obj.o_unknown2 <- x
+   | 3 -> obj.o_unknown1 <- x
+   | 4 -> obj.o_identifiers <- x
+   | _ -> failwith "assign to unknown compound position" 
+ 
+ let rec handle_bracket_list al i obj = 
+   let ss = (the_game ()).Load.script_style in
+   match al,i with
+   | [],_ -> ()
+   | (Bracket_Integer(x)::tl),i -> assign_bracket x i obj ss ;
+       handle_bracket_list tl (i+1) obj
+   | (Bracket_Symbol(s)::tl),i ->
+       begin
+         try
+           let x = int_of_sym (the_game ())
+             (ids_file_of_object_position ss i) s in
+           assign_bracket x i obj ss ;
+           handle_bracket_list tl (i+1) obj
+         with Not_found ->
+           parse_error (Printf.sprintf
+             "unknown object %s specifier [%s]"
+             (ids_file_of_object_position ss i) s )
+       end
+ 
+ let rec handle_compound_or_bracket obj cpos_ref arg = match arg with
+   BA_Rect (arg,rect) ->
+ 			handle_compound_or_bracket obj cpos_ref arg;
+ 			(match rect with
+ 			| None -> ()
+ 			| Some(x) -> obj.o_rect <- x);
+   | BA_Bracket l -> 
+       handle_bracket_list l 0 obj  
+   | BA_Compound (s,next_arg) -> 
+       begin 
+         let x = int_of_sym (the_game ()) "OBJECT" s in 
+         handle_compound_or_bracket obj (cpos_ref) next_arg ;
+         assign_compound x !cpos_ref obj ;
+         decr cpos_ref 
+       end 
+   | BA_Symbol s -> 
+         obj.o_identifiers <- int_of_sym (the_game ()) "OBJECT" s ;
+         decr cpos_ref
+ 
+   | BA_Integer i -> parse_error (Printf.sprintf "expecting compound object specifier or [bracket.object.specifier], got integer %ld" i)
+   | BA_String s -> 
+         obj.o_name <- s 
+   (* parse_error (Printf.sprintf "expecting compound object specifier or [bracket.object.specifier], got string \"%s\"" s) *)
+ 
+ let rec verify_arg_list name al fl = match (al,fl) with 
+     [],[] -> []
+   | al,[] ->
+   ( try parse_error (Printf.sprintf "Too many arguments to [%s]. Recovering." name) 
+     with _ -> [] ) 
+   | [],fl -> 
+   ( try parse_error (Printf.sprintf "Not enough arguments to [%s]. Recovering." name)
+   with _ -> [] )
+   | (ah::at),(fh::ft) -> 
+       begin 
+       let ah' = match ah with
+       |	BA_Rect(f,s) -> f
+       |	_ -> ah
+       in
+       let act = (match ah',fh.arg_kind with
+       | (BA_Integer i), Arg_Integer -> Act_Integer(i)
+ 
+ 
+       | (BA_Bracket([(Bracket_Symbol s)])), Arg_Integer
+       | (BA_String s), Arg_Integer 
+       | (BA_Symbol s), Arg_Integer when fh.arg_file <> "" -> 
+           begin
+             try
+               Act_Integer(int_of_sym (the_game ()) fh.arg_file s)
+             with _ -> 
+               begin
+                 try
+                   Act_Integer(Int32.of_string s)
+                 with _ -> begin
+                 (try parse_error (Printf.sprintf 
+                   "[%s] argument [%s] not found in [%s.IDS]" name s fh.arg_file)
+                  with _ -> ()) ; Act_Integer(Int32.zero) 
+                   end
+               end 
+           end
+ 
+       | (BA_String s), Arg_Integer ->
+             begin
+               try Act_Integer(Int32.of_string s)
+               with _ -> Act_Integer(Int32.zero) 
+             end 
+ 
+       | (BA_String s), Arg_String -> Act_String(s) 
+ 
+       | (BA_String s), Arg_Object ->
+         let tmp = empty_object_param () in
+         tmp.o_name <- s; 
+         Act_Object(tmp)
+ 
+       | (BA_Bracket _), Arg_Object 
+       | (BA_Symbol _), Arg_Object 
+       | (BA_Compound _), Arg_Object -> 
+         let obj = empty_object_param () in 
+         handle_compound_or_bracket obj (ref 4) ah ;
+         Act_Object(obj) 
+ 
+       | (BA_Bracket([Bracket_Integer(x);Bracket_Integer(y)])), 
+             Arg_Point -> Act_Point(x,y)
+ 
+       | (BA_Symbol s), Arg_String ->
+         begin
+           try parse_error (Printf.sprintf "Type mismatch in \"%s\" argument of [%s].\n\tExpecting type \"%s\". Recovering." fh.arg_comment name (print_arg_kind fh.arg_kind))
+           with _ -> Act_String(s)
+         end
+ 
+       | _, _ -> 
+         parse_error (Printf.sprintf "Type mismatch in \"%s\" argument of [%s].\n\tExpecting type \"%s\"." fh.arg_comment name (print_arg_kind fh.arg_kind))
+       ) in
+       (fh.arg_kind,act) :: (verify_arg_list name at ft)
+       end 
+ # 270 "src/bafparser.ml"
+ let yytransl_const = [|
+   257 (* LPAREN *);
+   258 (* RPAREN *);
+   259 (* LBRACKET *);
+   260 (* RBRACKET *);
+   261 (* PERIOD *);
+   262 (* COMMA *);
+   263 (* ACTIONOVERRIDE *);
+   264 (* NOT *);
+     0 (* EOF *);
+   265 (* IF *);
+   266 (* THEN *);
+   267 (* END *);
+   268 (* RESPONSE *);
+   269 (* ANYONE *);
+     0|]
+ 
+ let yytransl_block = [|
+   270 (* STRING *);
+   271 (* SYMBOL *);
+   272 (* TILDE_STRING *);
+   273 (* INTEGER *);
+   274 (* TRANS_REF *);
+     0|]
+ 
+ let yylhs = "\255\255\
+ \001\000\004\000\004\000\005\000\002\000\002\000\007\000\009\000\
+ \009\000\011\000\011\000\012\000\012\000\013\000\013\000\010\000\
+ \010\000\010\000\010\000\010\000\010\000\010\000\014\000\014\000\
+ \014\000\015\000\015\000\017\000\017\000\016\000\016\000\016\000\
+ \008\000\008\000\006\000\006\000\018\000\003\000\003\000\020\000\
+ \020\000\019\000\019\000\021\000\000\000\000\000\000\000"
+ 
+ let yylen = "\002\000\
+ \001\000\000\000\002\000\005\000\000\000\002\000\005\000\000\000\
+ \002\000\000\000\003\000\000\000\003\000\000\000\009\000\003\000\
+ \002\000\002\000\002\000\004\000\001\000\004\000\000\000\002\000\
+ \003\000\000\000\002\000\000\000\003\000\001\000\001\000\001\000\
+ \000\000\001\000\000\000\002\000\003\000\000\000\002\000\000\000\
+ \001\000\001\000\006\000\004\000\002\000\002\000\002\000"
+ 
+ let yydefred = "\000\000\
+ \000\000\000\000\000\000\000\000\000\000\045\000\001\000\000\000\
+ \034\000\046\000\000\000\000\000\000\000\000\000\047\000\000\000\
+ \042\000\000\000\003\000\006\000\000\000\000\000\000\000\039\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\021\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\031\000\
+ \032\000\030\000\000\000\000\000\000\000\018\000\000\000\000\000\
+ \000\000\000\000\017\000\000\000\044\000\000\000\009\000\000\000\
+ \004\000\036\000\007\000\000\000\000\000\027\000\000\000\024\000\
+ \000\000\016\000\000\000\000\000\000\000\000\000\037\000\022\000\
+ \000\000\000\000\025\000\013\000\020\000\041\000\043\000\011\000\
+ \029\000\000\000\000\000\000\000\000\000\000\000\015\000"
+ 
+ let yydgoto = "\004\000\
+ \006\000\010\000\015\000\007\000\008\000\037\000\011\000\012\000\
+ \034\000\035\000\055\000\050\000\046\000\048\000\043\000\044\000\
+ \062\000\038\000\016\000\079\000\017\000"
+ 
+ let yysindex = "\051\000\
+ \001\255\005\255\000\255\000\000\005\255\000\000\000\000\001\255\
+ \000\000\000\000\005\255\015\255\031\255\048\255\000\000\000\255\
+ \000\000\006\255\000\000\000\000\054\255\003\255\003\255\000\000\
+ \047\255\003\255\024\255\053\255\059\255\058\255\053\255\000\000\
+ \056\255\061\255\060\255\050\255\057\255\047\255\062\255\000\000\
+ \000\000\000\000\065\255\066\255\055\255\000\000\009\255\053\255\
+ \063\255\049\255\000\000\064\255\000\000\003\255\000\000\000\255\
+ \000\000\000\000\000\000\053\255\024\255\000\000\068\255\000\000\
+ \072\255\000\000\071\255\058\255\074\255\060\255\000\000\000\000\
+ \066\255\067\255\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\075\255\069\255\076\255\070\255\073\255\000\000"
+ 
+ let yyrindex = "\000\000\
+ \070\000\002\000\082\000\000\000\021\255\000\000\000\000\070\000\
+ \000\000\000\000\004\000\000\000\000\000\000\000\000\000\009\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\081\255\000\000\
+ \077\255\081\255\085\255\036\255\042\255\027\255\036\255\000\000\
+ \000\000\000\000\083\255\000\000\000\000\077\255\000\000\000\000\
+ \000\000\000\000\000\000\086\255\000\000\000\000\000\000\036\255\
+ \000\000\044\255\000\000\000\000\000\000\000\000\000\000\046\255\
+ \000\000\000\000\000\000\036\255\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\045\255\001\000\083\255\000\000\000\000\
+ \086\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000"
+ 
+ let yygindex = "\000\000\
+ \000\000\003\000\240\255\083\000\000\000\054\000\000\000\000\000\
+ \067\000\237\255\024\000\027\000\230\255\000\000\000\000\035\000\
+ \025\000\000\000\000\000\000\000\045\000"
+ 
+ let yytablesize = 277
+ let yytable = "\024\000\
+ \040\000\005\000\033\000\005\000\051\000\027\000\013\000\018\000\
+ \038\000\005\000\064\000\027\000\009\000\020\000\014\000\025\000\
+ \028\000\029\000\030\000\031\000\032\000\066\000\028\000\029\000\
+ \030\000\031\000\032\000\065\000\012\000\021\000\005\000\022\000\
+ \012\000\072\000\070\000\033\000\040\000\014\000\041\000\071\000\
+ \042\000\014\000\012\000\023\000\023\000\019\000\012\000\023\000\
+ \023\000\019\000\012\000\001\000\002\000\003\000\026\000\045\000\
+ \038\000\038\000\036\000\047\000\049\000\052\000\053\000\059\000\
+ \068\000\054\000\056\000\057\000\060\000\002\000\061\000\063\000\
+ \074\000\075\000\076\000\078\000\087\000\067\000\014\000\083\000\
+ \085\000\038\000\008\000\082\000\010\000\084\000\086\000\035\000\
+ \026\000\028\000\019\000\058\000\039\000\080\000\077\000\073\000\
+ \069\000\081\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
+ \000\000\000\000\000\000\000\000\000\000\000\000\000\000\040\000\
+ \000\000\000\000\000\000\040\000\040\000\005\000\000\000\040\000\
+ \033\000\000\000\033\000\038\000\038\000"
+ 
+ let yycheck = "\016\000\
+ \000\000\000\000\022\000\000\000\031\000\003\001\007\001\005\000\
+ \000\000\009\001\002\001\003\001\008\001\011\000\015\001\010\001\
+ \014\001\015\001\016\001\017\001\018\001\048\000\014\001\015\001\
+ \016\001\017\001\018\001\047\000\002\001\015\001\010\001\001\001\
+ \006\001\060\000\054\000\015\001\013\001\002\001\015\001\056\000\
+ \017\001\006\001\016\001\002\001\003\001\002\001\002\001\006\001\
+ \001\001\006\001\006\001\001\000\002\000\003\000\001\001\003\001\
+ \011\001\012\001\012\001\001\001\003\001\006\001\002\001\002\001\
+ \016\001\006\001\017\001\011\001\004\001\000\000\005\001\017\001\
+ \005\001\002\001\004\001\002\001\004\001\015\001\015\001\005\001\
+ \005\001\000\000\002\001\017\001\002\001\017\001\017\001\011\001\
+ \004\001\004\001\008\000\038\000\026\000\070\000\068\000\061\000\
+ \052\000\073\000\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
+ \255\255\255\255\255\255\255\255\255\255\255\255\255\255\007\001\
+ \255\255\255\255\255\255\011\001\012\001\010\001\255\255\015\001\
+ \015\001\255\255\015\001\011\001\012\001"
+ 
+ let yynames_const = "\
+   LPAREN\000\
+   RPAREN\000\
+   LBRACKET\000\
+   RBRACKET\000\
+   PERIOD\000\
+   COMMA\000\
+   ACTIONOVERRIDE\000\
+   NOT\000\
+   EOF\000\
+   IF\000\
+   THEN\000\
+   END\000\
+   RESPONSE\000\
+   ANYONE\000\
+   "
+ 
+ let yynames_block = "\
+   STRING\000\
+   SYMBOL\000\
+   TILDE_STRING\000\
+   INTEGER\000\
+   TRANS_REF\000\
+   "
+ 
+ let yyact = [|
+   (fun _ -> failwith "parser")
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 0 : 'ifblock_list) in
+     Obj.repr(
+ # 264 "src/bafparser.mly"
+                                 ( _1 )
+ # 468 "src/bafparser.ml"
+                : Bcs.script))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 267 "src/bafparser.mly"
+                                 ( [] )
+ # 474 "src/bafparser.ml"
+                : 'ifblock_list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'ifblock) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'ifblock_list) in
+     Obj.repr(
+ # 268 "src/bafparser.mly"
+                                 ( _1 :: _2 )
+ # 482 "src/bafparser.ml"
+                : 'ifblock_list))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 3 : Bcs.trigger list) in
+     let _4 = (Parsing.peek_val __caml_parser_env 1 : 'response_list) in
+     Obj.repr(
+ # 271 "src/bafparser.mly"
+                                                  ( (_2,_4) )
+ # 490 "src/bafparser.ml"
+                : 'ifblock))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 274 "src/bafparser.mly"
+                                 ( [] )
+ # 496 "src/bafparser.ml"
+                : Bcs.trigger list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'trigger) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : Bcs.trigger list) in
+     Obj.repr(
+ # 275 "src/bafparser.mly"
+                                 ( _1 :: _2 )
+ # 504 "src/bafparser.ml"
+                : Bcs.trigger list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 4 : 'opt_not) in
+     let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
+     let _4 = (Parsing.peek_val __caml_parser_env 1 : 'arg_list) in
+     Obj.repr(
+ # 279 "src/bafparser.mly"
+   (
+     let ids = 
+       try ids_of_sym (the_game ()) "TRIGGER" _2 
+       with _ -> 
+       parse_error (Printf.sprintf "[%s] not found in TRIGGER.IDS" _2)
+     in 
+     let act_list = ref (verify_arg_list _2 _4 ids.i_args) in 
+     let ss = (the_game ()).Load.script_style in 
+     (match is_concat_string ss ids with
+     | 1 -> fixup_concat act_list 
+     | 2 -> fixup_concat2 act_list
+     | _ -> () ) ;
+     let t1 = get_next_int act_list in
+     let t2 = get_next_int act_list in 
+     let t3 = get_next_string act_list in
+     let t4 = get_next_string act_list in
+     let t5 = get_next_object act_list in
+     let c = get_next_point act_list in 
+     let unknown = get_next_int act_list in 
+     { trigger_id = ids.i_num ; negated = _1 ;
+       t_1 = t1; t_2 = t2; unknown = unknown ; t_3 = t3; t_4 = t4; t_5 = t5; 
+       t_coord = c ; (* FIXME! *) 
+     } 
+   )
+ # 536 "src/bafparser.ml"
+                : 'trigger))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 305 "src/bafparser.mly"
+             ( [] )
+ # 542 "src/bafparser.ml"
+                : 'arg_list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'arg) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'comma_before_arg_list) in
+     Obj.repr(
+ # 306 "src/bafparser.mly"
+                             ( _1 :: _2 )
+ # 550 "src/bafparser.ml"
+                : 'arg_list))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 309 "src/bafparser.mly"
+                           ( [] )
+ # 556 "src/bafparser.ml"
+                : 'comma_before_arg_list))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : 'arg) in
+     let _3 = (Parsing.peek_val __caml_parser_env 0 : 'comma_before_arg_list) in
+     Obj.repr(
+ # 310 "src/bafparser.mly"
+                                   ( _2 :: _3 )
+ # 564 "src/bafparser.ml"
+                : 'comma_before_arg_list))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 313 "src/bafparser.mly"
+                            ( "" )
+ # 570 "src/bafparser.ml"
+                : 'sound_opt))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
+     Obj.repr(
+ # 314 "src/bafparser.mly"
+                            ( _2 )
+ # 577 "src/bafparser.ml"
+                : 'sound_opt))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 317 "src/bafparser.mly"
+                                           ( None )
+ # 583 "src/bafparser.ml"
+                : 'rect_opt))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 7 : Int32.t) in
+     let _4 = (Parsing.peek_val __caml_parser_env 5 : Int32.t) in
+     let _6 = (Parsing.peek_val __caml_parser_env 3 : Int32.t) in
+     let _8 = (Parsing.peek_val __caml_parser_env 1 : Int32.t) in
+     Obj.repr(
+ # 318 "src/bafparser.mly"
+                                                                          ( Some(_2,_4,_6,_8) )
+ # 593 "src/bafparser.ml"
+                : 'rect_opt))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 2 : string) in
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : 'opt_arg) in
+     let _3 = (Parsing.peek_val __caml_parser_env 0 : 'rect_opt) in
+     Obj.repr(
+ # 321 "src/bafparser.mly"
+                                 ( BA_Rect((match _2 with 
+                     None -> BA_Symbol(_1) 
+                   | Some(a) -> BA_Compound(_1,a)),_3) )
+ # 604 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : Int32.t) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'rect_opt) in
+     Obj.repr(
+ # 324 "src/bafparser.mly"
+                    ( BA_Rect(BA_Integer(_1),_2) )
+ # 612 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'rect_opt) in
+     Obj.repr(
+ # 325 "src/bafparser.mly"
+                   ( BA_Rect(BA_String(_1),_2) )
+ # 620 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'sound_opt) in
+     Obj.repr(
+ # 327 "src/bafparser.mly"
+   ( let result = Dlg.Local_String({ lse_male = _1 ; lse_male_sound = _2; 
+                     lse_female = _1; lse_female_sound = _2;}) in 
+     (match !Dlg.local_string_ht with
+       Some(l) -> if not (List.mem result l ) then 
+           Dlg.local_string_ht := Some(result :: l )
+     | _ -> () ) ;
+     match Dc.resolve_string_while_loading result with
+       Dlg.TLK_Index(i) -> BA_Integer(Int32.of_int i)
+     | _ -> 
+       if !Dc.doing_traify then BA_Integer(Int32.zero) else 
+       parse_error (Printf.sprintf "unable to resolve TLK string ~%s~" _1)
+   )
+ # 639 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 3 : string) in
+     let _2 = (Parsing.peek_val __caml_parser_env 2 : 'sound_opt) in
+     let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
+     let _4 = (Parsing.peek_val __caml_parser_env 0 : 'sound_opt) in
+     Obj.repr(
+ # 340 "src/bafparser.mly"
+   ( let result = Dlg.Local_String({ lse_male = _1 ; lse_male_sound = _2; 
+           lse_female = _3; lse_female_sound = _4; }) in 
+     (match !Dlg.local_string_ht with
+       Some(l) -> if not (List.mem result l) then 
+         Dlg.local_string_ht := Some(result :: l)
+     | _ -> () ) ;
+     match Dc.resolve_string_while_loading result with
+       Dlg.TLK_Index(i) -> BA_Integer(Int32.of_int i)
+     | _ -> 
+       if !Dc.doing_traify then BA_Integer(Int32.zero) else 
+       parse_error (Printf.sprintf "unable to resolve TLK string ~%s~" _1)
+   )
+ # 660 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 0 : Int32.t) in
+     Obj.repr(
+ # 353 "src/bafparser.mly"
+   ( match Dc.resolve_string_while_loading (Dlg.Trans_String(Int32.to_int _1))
+     with Dlg.TLK_Index(i) -> BA_Integer(Int32.of_int i)
+     | _ -> parse_error (Printf.sprintf "unable to resolve translation string @%ld" _1)
+   )
+ # 670 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 2 : 'ba_arg_list) in
+     let _4 = (Parsing.peek_val __caml_parser_env 0 : 'rect_opt) in
+     Obj.repr(
+ # 357 "src/bafparser.mly"
+                                          ( BA_Rect(BA_Bracket(_2),_4) )
+ # 678 "src/bafparser.ml"
+                : 'arg))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 360 "src/bafparser.mly"
+                 ( None )
+ # 684 "src/bafparser.ml"
+                : 'opt_arg))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 361 "src/bafparser.mly"
+                 ( Some(BA_Symbol("Myself")) )
+ # 690 "src/bafparser.ml"
+                : 'opt_arg))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : 'arg) in
+     Obj.repr(
+ # 362 "src/bafparser.mly"
+                     ( Some(_2) )
+ # 697 "src/bafparser.ml"
+                : 'opt_arg))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 365 "src/bafparser.mly"
+                 ( [] )
+ # 703 "src/bafparser.ml"
+                : 'ba_arg_list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'ba_arg) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'period_before_ba_arg_list) in
+     Obj.repr(
+ # 366 "src/bafparser.mly"
+                                    ( _1 :: _2 )
+ # 711 "src/bafparser.ml"
+                : 'ba_arg_list))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 369 "src/bafparser.mly"
+                               ( [] )
+ # 717 "src/bafparser.ml"
+                : 'period_before_ba_arg_list))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : 'ba_arg) in
+     let _3 = (Parsing.peek_val __caml_parser_env 0 : 'period_before_ba_arg_list) in
+     Obj.repr(
+ # 370 "src/bafparser.mly"
+                                           ( _2 :: _3 )
+ # 725 "src/bafparser.ml"
+                : 'period_before_ba_arg_list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 0 : Int32.t) in
+     Obj.repr(
+ # 373 "src/bafparser.mly"
+                  ( Bracket_Integer(_1) )
+ # 732 "src/bafparser.ml"
+                : 'ba_arg))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 374 "src/bafparser.mly"
+                 ( Bracket_Integer(Int32.zero) )
+ # 738 "src/bafparser.ml"
+                : 'ba_arg))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
+     Obj.repr(
+ # 375 "src/bafparser.mly"
+                 ( Bracket_Symbol(_1) )
+ # 745 "src/bafparser.ml"
+                : 'ba_arg))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 378 "src/bafparser.mly"
+             ( false )
+ # 751 "src/bafparser.ml"
+                : 'opt_not))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 379 "src/bafparser.mly"
+             ( true )
+ # 757 "src/bafparser.ml"
+                : 'opt_not))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 382 "src/bafparser.mly"
+                   ( [] )
+ # 763 "src/bafparser.ml"
+                : 'response_list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'response) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : 'response_list) in
+     Obj.repr(
+ # 383 "src/bafparser.mly"
+                          ( _1 :: _2 )
+ # 771 "src/bafparser.ml"
+                : 'response_list))
+ ; (fun __caml_parser_env ->
+     let _2 = (Parsing.peek_val __caml_parser_env 1 : Int32.t) in
+     let _3 = (Parsing.peek_val __caml_parser_env 0 : Bcs.action list) in
+     Obj.repr(
+ # 386 "src/bafparser.mly"
+                                         ( ((Int32.to_int _2),_3) )
+ # 779 "src/bafparser.ml"
+                : 'response))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 389 "src/bafparser.mly"
+                   ( [] )
+ # 785 "src/bafparser.ml"
+                : Bcs.action list))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 1 : 'complex_action) in
+     let _2 = (Parsing.peek_val __caml_parser_env 0 : Bcs.action list) in
+     Obj.repr(
+ # 390 "src/bafparser.mly"
+                              ( _1 :: _2 )
+ # 793 "src/bafparser.ml"
+                : Bcs.action list))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 393 "src/bafparser.mly"
+                                ( )
+ # 799 "src/bafparser.ml"
+                : 'not_actually_optional_rparen))
+ ; (fun __caml_parser_env ->
+     Obj.repr(
+ # 394 "src/bafparser.mly"
+          ( )
+ # 805 "src/bafparser.ml"
+                : 'not_actually_optional_rparen))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 0 : 'action) in
+     Obj.repr(
+ # 397 "src/bafparser.mly"
+                         ( _1 )
+ # 812 "src/bafparser.ml"
+                : 'complex_action))
+ ; (fun __caml_parser_env ->
+     let _3 = (Parsing.peek_val __caml_parser_env 3 : 'arg) in
+     let _5 = (Parsing.peek_val __caml_parser_env 1 : 'action) in
+     let _6 = (Parsing.peek_val __caml_parser_env 0 : 'not_actually_optional_rparen) in
+     Obj.repr(
+ # 399 "src/bafparser.mly"
+   ( let base_action = _5 in 
+     let act_list = ref (verify_arg_list "ActionOverride" [_3] 
+       [ { arg_kind = Arg_Object ; arg_comment = "Actor" ; arg_file = "" } ])
+     in 
+     let actor = get_next_object act_list in
+     base_action.a_1 <- actor ;
+     base_action 
+   )
+ # 828 "src/bafparser.ml"
+                : 'complex_action))
+ ; (fun __caml_parser_env ->
+     let _1 = (Parsing.peek_val __caml_parser_env 3 : string) in
+     let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arg_list) in
+     Obj.repr(
+ # 410 "src/bafparser.mly"
+   (
+     let ids = 
+       try ids_of_sym (the_game ()) "ACTION" _1
+       with _ -> 
+       parse_error (Printf.sprintf "[%s] not found in ACTION.IDS" _1)
+     in 
+     let act_list = ref (verify_arg_list _1 _3 ids.i_args) in 
+     let ss = (the_game ()).Load.script_style in 
+     (match is_concat_string ss ids with
+     | 1 -> fixup_concat act_list ; 
+     | 2 -> fixup_concat2 act_list ; 
+     | _ -> () ) ; 
+     let a1 = empty_object_param() in 
+     let a2 = get_next_object act_list in
+     let a3 = get_next_object act_list in
+     let a4 = get_next_int act_list in
+     let a5 = get_next_point act_list in
+     let a6 = get_next_int act_list in
+     let a7 = get_next_int act_list in
+     let a8 = get_next_string act_list in
+     let a9 = get_next_string act_list in
+     {
+       action_id = ids.i_num ;
+       a_1 = a1; a_2 = a2; a_3 = a3; a_4 = a4; a_5 = a5;
+       a_6 = a6; a_7 = a7; a_8 = a8; a_9 = a9; 
+     } 
+   )
+ # 862 "src/bafparser.ml"
+                : 'action))
+ (* Entry baf_file *)
+ ; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
+ (* Entry trigger_list *)
+ ; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
+ (* Entry action_list *)
+ ; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
+ |]
+ let yytables =
+   { Parsing.actions=yyact;
+     Parsing.transl_const=yytransl_const;
+     Parsing.transl_block=yytransl_block;
+     Parsing.lhs=yylhs;
+     Parsing.len=yylen;
+     Parsing.defred=yydefred;
+     Parsing.dgoto=yydgoto;
+     Parsing.sindex=yysindex;
+     Parsing.rindex=yyrindex;
+     Parsing.gindex=yygindex;
+     Parsing.tablesize=yytablesize;
+     Parsing.table=yytable;
+     Parsing.check=yycheck;
+     Parsing.error_function=parse_error;
+     Parsing.names_const=yynames_const;
+     Parsing.names_block=yynames_block }
+ let baf_file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
+    (Parsing.yyparse yytables 1 lexfun lexbuf : Bcs.script)
+ let trigger_list (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
+    (Parsing.yyparse yytables 2 lexfun lexbuf : Bcs.trigger list)
+ let action_list (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
+    (Parsing.yyparse yytables 3 lexfun lexbuf : Bcs.action list)
