*** WeiDU-185/src/util.ml	2005-06-27 01:40:07.000000000 +0200
--- WeiDU/src/util.ml	2007-07-16 13:39:11.000000000 +0200
***************
*** 1,3 ****
--- 1,11 ----
+ (* Note added due to LGPL terms.
+ 
+ This file was edited by Valerio Bigiani, AKA The Bigg, starting from
+ 6 November 2005. All changes for this file are listed in
+ diffs/src.util.ml.diff file, as the output of a diff -Bw -c -N command.
+ 
+ It was originally taken from Westley Weimer's WeiDU 185. *)
+ 
  (* generic utilities *)
  
  open Arch
***************
*** 10,17 ****
--- 18,41 ----
    lse_female_sound : string ;
  }
  
+ let one_newline_regexp = Str.regexp "[\n]"
+ let one_newline_or_cr_regexp = Str.regexp "[\r\n]"
+ let many_newline_or_cr_regexp = Str.regexp "[\r\n]+"
+ let many_whitespace_regexp = Str.regexp "[ \t]+"
+ let many_not_whitespace_regexp = Str.regexp "[^ \t\n\r]+"
+ 
+ let value_of_option x = match x with
+   | Some(s) -> s
+   | None -> failwith "value_of_option None"
+ 
  let log_channel = ref None
  
+ let debug_ocaml = ref false
+ 
+ let debug_modder = ref true
+ 
+ let be_silent = ref false
+ 
  let log_or_print fmt =
    let k result =
      match !log_channel with
***************
*** 20,25 ****
--- 44,57 ----
    in
    Printf.kprintf k fmt
  
+ let log_or_print_modder fmt =
+   let k result =
+     if !debug_modder then match !log_channel with
+       None -> print_string result ; flush stdout
+     | Some(o) -> output_string o result ; flush o
+   in
+   Printf.kprintf k fmt
+ 
  let log_only fmt =
    let k result =
      match !log_channel with
***************
*** 28,36 ****
    in 
    Printf.kprintf k fmt 
  
  let log_and_print fmt = 
    let k result = begin
!     output_string stdout result ; flush stdout ;
      match !log_channel with
        None -> ()
      | Some(o) -> output_string o result ; flush o 
--- 60,76 ----
    in
    Printf.kprintf k fmt
  
+ let log_only_modder fmt =
+   let k result =
+     if !debug_modder then match !log_channel with
+       None -> ()
+     | Some(o) -> output_string o result ; flush o
+   in
+   Printf.kprintf k fmt
+ 
  let log_and_print fmt =
    let k result = begin
!     if not !be_silent then (output_string stdout result ; flush stdout );
      match !log_channel with
        None -> ()
      | Some(o) -> output_string o result ; flush o
***************
*** 38,63 ****
    in 
    Printf.kprintf k fmt 
  
  let inlined_files = Hashtbl.create 15 
  
  let backup_ht = Hashtbl.create 511
  let backup_dir = ref None 
  let backup_list_chn = ref None
  
  let set_backup_dir str i =
    let i = Printf.sprintf "%d" i in 
    let backup_dir_name = Printf.sprintf "%s/%s" str i in
!   (try
!     Unix.mkdir backup_dir_name 511 (* 511 = octal 0777 = a+rwx *)
!   with _ -> ()) ;
    backup_dir := Some(backup_dir_name) ; 
    (match !backup_list_chn with
      Some(c) -> close_out c
    | None -> ()) ; 
    let backup_filename = (backup_dir_name ^ "/UNINSTALL." ^ i) in 
    Hashtbl.clear backup_ht ; 
    (try 
!     backup_list_chn := Some(open_out_bin backup_filename)
    with e -> 
      log_and_print "WARNING: unable to open [%s]: %s
  Will be unable to UNINSTALL later.\n" backup_filename (Printexc.to_string e))
--- 78,138 ----
    in
    Printf.kprintf k fmt
  
+ let log_and_print_modder fmt =
+   let k result = begin
+     if not !be_silent && !debug_modder then (output_string stdout result ; flush stdout );
+     if !debug_modder then match !log_channel with
+       None -> ()
+     | Some(o) -> output_string o result ; flush o
+   end
+   in
+   Printf.kprintf k fmt
+ 
+ let recursive_mkdir directory mode =
+   let dir_split = Str.split (Str.regexp "[/\\]") directory in
+   let added_up_dir = ref "" in
+   let skip_first_slash = ref false in
+   if String.get directory 0 = '\\' || String.get directory 0 = '/' then skip_first_slash := true ;
+   List.iter ( fun part ->
+       added_up_dir := !added_up_dir ^ (if !skip_first_slash then "/" else "") ^ part ;
+       skip_first_slash := true ;
+ (* 	  log_and_print "MKDIR %s\n" !added_up_dir; *)
+ 	  try
+ 	    Case_ins.unix_mkdir !added_up_dir mode;
+ 	  with e -> (
+ 	    match e with
+ 	    | Unix.Unix_error(Unix.EEXIST,_,_) -> ()
+ 	    | _ -> log_and_print "Problem %s on %s: util.ml\n" (Printexc.to_string e) !added_up_dir ;
+     )
+   ) dir_split
+ 
  let inlined_files = Hashtbl.create 15
+ ;;
+ 
+ List.iter ( fun (name,contents) ->
+     Hashtbl.add inlined_files name contents
+   ) Tph.list_of_stuff
+ ;;
  
  let backup_ht = Hashtbl.create 511
  let backup_dir = ref None
  let backup_list_chn = ref None
+ let mappings_list_chn = ref None
  
  let set_backup_dir str i =
    let i = Printf.sprintf "%d" i in 
    let backup_dir_name = Printf.sprintf "%s/%s" str i in
!   recursive_mkdir backup_dir_name 511; (* 511 = octal 0777 = a+rwx *)
    backup_dir := Some(backup_dir_name) ;
    (match !backup_list_chn with
      Some(c) -> close_out c
    | None -> ()) ; 
    let backup_filename = (backup_dir_name ^ "/UNINSTALL." ^ i) in
+   let mappings_filename = (backup_dir_name ^ "/MAPPINGS." ^ i) in
    Hashtbl.clear backup_ht ; 
    (try
!     backup_list_chn := Some(Case_ins.perv_open_out_bin backup_filename);
!     mappings_list_chn := Some(Case_ins.perv_open_out_bin mappings_filename);
    with e -> 
      log_and_print "WARNING: unable to open [%s]: %s
  Will be unable to UNINSTALL later.\n" backup_filename (Printexc.to_string e))
***************
*** 69,76 ****
    try
      let out = 
        if !append_to_log then 
!         open_out_gen [ Open_append ; Open_creat ; Open_text ] 0o777 filename 
!       else open_out filename 
      in
      log_channel := Some(out) ;
      log_file := filename ;
--- 144,151 ----
    try
      let out = 
        if !append_to_log then 
!         Case_ins.perv_open_out_gen [ Open_append ; Open_creat ; Open_text ] 0o777 filename 
!       else Case_ins.perv_open_out filename 
      in
      log_channel := Some(out) ;
      log_file := filename ;
***************
*** 230,267 ****
    done 
  
  
- let load_file name =
-   if Hashtbl.mem inlined_files name then
-     Hashtbl.find inlined_files name
-   else 
-   try begin
-     Stats.time "loading files" (fun () -> 
-     let stats = Unix.stat name in
-     let size = stats.Unix.st_size in 
-     if size = 0 then 
-       log_or_print "WARNING: [%s] is a 0 byte file\n" name 
-     else if size < 0 then begin
-       log_and_print "ERROR: [%s] has reported size %d\n" name size ;
-       failwith ("error loading " ^ name)
-     end else if size > Sys.max_string_length then begin
-       log_and_print "ERROR: [%s] has size %d: TOO BIG FOR WEIDU (max %d)\n" 
-           name size Sys.max_string_length;
-       failwith ("error loading " ^ name)
-     end ; 
-     let buff = String.make size '\000' in
-     let fd = Unix.openfile name [Unix.O_RDONLY] 0 in
-     my_read size fd buff name ; 
-     Unix.close fd ; 
-     log_only "[%s] loaded, %d bytes\n" name size ; 
-     buff) () 
-   end
-   with e -> 
-     log_and_print "ERROR: error loading [%s]\n" name ;
-     raise e
- 
  let file_size name =
    try 
!     let stats = Unix.stat name in
      stats.Unix.st_size 
    with _ ->  -1
  
--- 305,313 ----
    done 
  
  
  let file_size name =
    try 
!     let stats = Case_ins.unix_stat name in
      stats.Unix.st_size 
    with _ ->  -1
  
***************
*** 269,275 ****
  
  let is_directory name =
    try 
!     let stats = Unix.stat name in
      let res = stats.Unix.st_kind = Unix.S_DIR in
      (* log_only "%s is a directory: %b\n" name res ;  *) 
      res 
--- 315,321 ----
  
  let is_directory name =
    try
!     let stats = Case_ins.unix_stat name in
      let res = stats.Unix.st_kind = Unix.S_DIR in
      (* log_only "%s is a directory: %b\n" name res ;  *) 
      res 
***************
*** 284,304 ****
      base,ext
    with _ -> name,""
  
! let list_of_files_in_directory d =
!   let result = ref [] in 
!   (try
!     let dh = Unix.opendir d in
      begin
        try 
!         while true do
!           result := Unix.readdir dh :: !result 
!         done 
!       with _ -> () 
!     end ;
!     Unix.closedir dh 
!   with _ -> ()) ; !result
  
! let backup_if_extant filename = 
    if Hashtbl.mem backup_ht 
                   (String.uppercase (slash_to_backslash filename)) then 
      () 
--- 330,345 ----
      base,ext
    with _ -> name,""
  
! let my_unlink file =
  	begin
  		try
! 			Case_ins.unix_unlink file
! 		with e ->
! 			log_or_print "Unable to Unlink [%s]: %s"
! 				file (Printexc.to_string e)
! 	end
  
! let rec backup_if_extant filename =
    if Hashtbl.mem backup_ht
                   (String.uppercase (slash_to_backslash filename)) then
      ()
***************
*** 311,337 ****
      | None -> ()
      ); 
      match !backup_dir with
!     | Some(dir) when file_exists filename -> 
!       let backup_name = dir ^ "/" ^ (Filename.basename filename) in
!       let buff = load_file filename in
!       let backup_chn = open_out_bin backup_name in
!       output_string backup_chn buff ;
!       close_out backup_chn ;
!       log_or_print "[%s] backed up to [%s]\n" filename backup_name 
      | _ -> () 
    end
  
  let open_for_writing_internal backup filename binary = 
    (if (backup) then backup_if_extant filename) ;
    if file_exists filename then (* if it already exists *)
      begin (* handle read-only files! *)
        try 
!         Unix.chmod filename 511 ; (* 511 = octal 0777 = a+rwx *)
        with e -> () 
          (* log_or_print "WARNING: chmod %s : %s\n" filename 
            (Printexc.to_string e) *)
      end ;
!   let out_chn = (if binary then open_out_bin else open_out) filename in
    out_chn 
  
  let open_for_writing = open_for_writing_internal true 
--- 352,474 ----
      | None -> ()
      );
      match !backup_dir with
!     | Some(dir) when file_exists filename -> (
!       let name = filename in
!       let out = dir ^ "/" ^ (Str.global_replace (Str.regexp "[\\/]") "." filename) in
!       let out1 = dir ^ "/" ^ Filename.basename filename in
!       let where = ref "" in
!       try
!         if file_exists out1 then
! 					where := out
! 				else
! 					where := out1
! 				;
! 				(
! 				match !mappings_list_chn with
! 					| Some(chn) -> output_string chn (filename ^ " " ^ !where ^ "\n"); flush chn
! 					| None -> ()
! 				);
! 				copy_large_file name !where "creating a backup"
!       with e ->
!         log_and_print "ERROR: error copying [%s]\n" name ;
!         raise e
!     )
!     | Some(dir) when not (file_exists filename) -> (
! 				match !mappings_list_chn with
! 					| Some(chn) -> output_string chn (filename ^ "\n"); flush chn
! 					| None -> ()
!     )
      | _ -> ()
    end
  
+ and copy_large_file name out reason =
+ (*  log_or_print "Copying a large file: %s to %s\n" name out; *)
+   try begin
+     Stats.time "loading files" (fun () ->
+       let stats = Case_ins.unix_stat name in
+       let size = stats.Unix.st_size in
+       if size = 0 then
+         log_or_print_modder "WARNING: [%s] is a 0 byte file\n" name
+       else if size < 0 then begin
+         log_and_print "ERROR: [%s] has reported size %d\n" name size ;
+         failwith ("error loading " ^ name)
+       end ;
+       if (my_unlink out ; true) then begin
+         let in_fd  = Case_ins.unix_openfile name [Unix.O_RDONLY] 0 in
+         let out_fd = Case_ins.unix_openfile out  [Unix.O_WRONLY ; Unix.O_CREAT] 511 in
+         let chunk_size = 10240 in
+         let chunk = String.create chunk_size in
+         let sofar = ref 0 in
+         while !sofar < size do
+           let chunk_size = min (size - !sofar) chunk_size in
+           my_read chunk_size in_fd chunk name ;
+           my_write chunk_size out_fd chunk out ;
+           sofar := !sofar + chunk_size ;
+         done ;
+         Unix.close in_fd ;
+         Unix.close out_fd ;
+         log_only "%s copied to %s, %d bytes\n" name out size ;
+       end
+     ) ()
+   end
+   with e ->
+     log_and_print "ERROR: error copying [%s]\n" name ;
+     raise e
+ 
+ let load_file name =
+   if Hashtbl.mem inlined_files name then
+     Hashtbl.find inlined_files (Arch.backslash_to_slash name)
+   else
+   try begin
+     Stats.time "loading files" (fun () ->
+     let stats = Case_ins.unix_stat name in
+     let size = stats.Unix.st_size in
+     if size = 0 then
+       log_or_print_modder "WARNING: [%s] is a 0 byte file\n" name
+     else if size < 0 then begin
+       log_and_print "ERROR: [%s] has reported size %d\n" name size ;
+       failwith ("error loading " ^ name)
+     end else if size > Sys.max_string_length then begin
+       log_and_print "ERROR: [%s] has size %d: TOO BIG FOR WEIDU (max %d)\n"
+           name size Sys.max_string_length;
+       failwith ("error loading " ^ name)
+     end ;
+     let buff = String.make size '\000' in
+     let fd = Case_ins.unix_openfile name [Unix.O_RDONLY] 0 in
+     my_read size fd buff name ;
+     Unix.close fd ;
+     log_only "[%s] loaded, %d bytes\n" name size ;
+     buff) ();
+   end
+   with e ->
+     log_and_print "ERROR: error loading [%s]\n" name ;
+     raise e
+ 
+ let list_of_files_in_directory d =
+   let result = ref [] in 
+   (try
+     let dh = Case_ins.unix_opendir d in
+     begin
+       try 
+         while true do
+           result := Unix.readdir dh :: !result 
+         done 
+       with _ -> () 
+     end ;
+     Unix.closedir dh 
+   with _ -> ()) ; !result
+ 
  let open_for_writing_internal backup filename binary =
    (if (backup) then backup_if_extant filename) ;
    if file_exists filename then (* if it already exists *)
      begin (* handle read-only files! *)
        try 
!         Case_ins.unix_chmod filename 511 ; (* 511 = octal 0777 = a+rwx *)
        with e -> () 
          (* log_or_print "WARNING: chmod %s : %s\n" filename 
            (Printexc.to_string e) *)
      end ;
!   let out_chn = (if binary then Case_ins.perv_open_out_bin else Case_ins.perv_open_out) filename in
    out_chn 
  
  let open_for_writing = open_for_writing_internal true
***************
*** 420,426 ****
    try
      Hashtbl.find error_chn_ht sort 
    with Not_found ->
!     let oc = open_out (Printf.sprintf "%s/%s" !error_chn_base sort) in
      Hashtbl.add error_chn_ht sort oc ;
      oc
  
--- 557,563 ----
    try
      Hashtbl.find error_chn_ht sort
    with Not_found ->
!     let oc = Case_ins.perv_open_out (Printf.sprintf "%s/%s" !error_chn_base sort) in
      Hashtbl.add error_chn_ht sort oc ;
      oc
  
***************
*** 463,469 ****
        let result = Stats.time sort_of_file 
          (fun () -> parse_lex_fun lexbuf) () in
        pop_context () ;
!       log_or_print "[%s] parsed\n" filename ; 
        result
      with e -> 
        (try input_error "" (Printexc.to_string e) with _ -> () ) ; 
--- 600,606 ----
        let result = Stats.time sort_of_file 
          (fun () -> parse_lex_fun lexbuf) () in
        pop_context () ;
!       log_or_print_modder "[%s] parsed\n" filename ; 
        result
      with e -> 
        (try input_error "" (Printexc.to_string e) with _ -> () ) ;
***************
*** 480,486 ****
          filename (Printexc.to_string e) ; 
        raise e 
    end else begin 
!     let inchan = open_in filename in 
      try 
        begin 
        let lexbuf : Lexing.lexbuf = lex_init filename inchan in
--- 617,623 ----
          filename (Printexc.to_string e) ; 
        raise e 
    end else begin 
!     let inchan = Case_ins.perv_open_in filename in 
      try 
        begin 
        let lexbuf : Lexing.lexbuf = lex_init filename inchan in
***************
*** 501,503 ****
--- 638,643 ----
  let return_value_error_argument = 4
  
  let return_value = ref return_value_success
+ 
+ let bool_xor a b =
+   ((a && not b) || (not a && b))
